// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/shader/UIShader.hpp"
#include "component/ui/UICanvas.hpp"

namespace Dawn {
  struct UIAlign {
    glm::vec2 position;
    glm::vec2 size;
  };

  class UIComponent {
    protected:
      /**
       * Virtual method overridden by the UIComponent to get the quads for the
       * component.
       * 
       * @param alignment The alignment of this component.
       * @param ctx The canvas to add the quads to.
       */
      virtual void getSelfQuads(
        const struct UIAlign alignment,
        UICanvas &ctx
      ) = 0;

      /**
       * Allows a component to modify its self alignment.
       * 
       * @param alignment Aliugnment to modify.
       * @return New alignment for this component.
       */
      virtual struct UIAlign getSelfAlignment(const struct UIAlign alignment);

    public:
      glm::vec2 position;

      /**
       * Virtual method overridden by the UIComponent to get the children of
       * this component.
       */
      virtual std::vector<std::shared_ptr<UIComponent>> getChildren();

      /**
       * Method called by the UICanvas to get the quads for this component.
       * 
       * @param alignment The alignment of this component.
       * @param ctx The canvas to add the quads to.
       */
      void getQuads(
        const struct UIAlign alignment,
        UICanvas &ctx
      );
  };
}